@@ -4,7 +4,10 @@ def sage_setup(distributions, *,
                spkgs=(),
                recurse_packages=('sage',),
                package_data=None,
-               cmdclass=None):
+               data_files=(),
+               cmdclass=None,
+               ext_modules=(),
+               py_limited_api=False):
     r"""
     Replacement for :func:`setuptools.setup` for building distribution packages of the Sage library
 
@@ -45,8 +48,9 @@ def sage_setup(distributions, *,
 
     # Work around a Cython problem in Python 3.8.x on macOS
     # https://github.com/cython/cython/issues/3262
-    import os
-    if os.uname().sysname == 'Darwin':
+
+    import platform
+    if platform.system() == 'Darwin':
         import multiprocessing
         multiprocessing.set_start_method('fork', force=True)
 
@@ -71,6 +75,7 @@ def sage_setup(distributions, *,
     import setuptools.command.egg_info
     setuptools.command.egg_info.walk_revctrl = lambda: ()
 
+    import os
     import sys
 
     from sage_setup.excepthook import excepthook
@@ -83,6 +88,8 @@ def sage_setup(distributions, *,
     cmdclass.update(dict(build_ext=sage_build_ext_minimal,
                          build_py=sage_build_py))
 
+    options = {}
+
     sdist = len(sys.argv) > 1 and (sys.argv[1] in ["sdist", "egg_info", "dist_info"])
 
     # ########################################################
@@ -109,6 +116,14 @@ def sage_setup(distributions, *,
         sage.env.default_required_modules = required_modules
         sage.env.default_optional_modules = optional_modules
 
+        extension_kwds = {}
+        if py_limited_api and os.environ.get('CIBUILDWHEEL', None) and sys.version_info >= (3, 12, 0, 0):
+            # https://cibuildwheel.pypa.io/en/stable/options/#examples_8
+            # https://cython.readthedocs.io/en/latest/src/userguide/limited_api.html#setuptools-and-setup-py
+            extension_kwds['define_macros'] = [("Py_LIMITED_API", 0x030C0000)]
+            extension_kwds['py_limited_api'] = True
+            options["bdist_wheel"] = {"py_limited_api": "cp312"}
+
         if interpreters:
             log.info("Generating auto-generated sources")
             # from sage_setup.autogen import autogen_all
@@ -122,7 +137,7 @@ def sage_setup(distributions, *,
         t = time.time()
 
         python_packages, python_modules, cython_modules = find_python_sources(
-            '.', recurse_packages, distributions=distributions)
+            '.', recurse_packages, distributions=distributions, extension_kwds=extension_kwds)
         extra_files = find_extra_files(
             '.', recurse_packages, '/doesnotexist', distributions=distributions)
 
@@ -173,6 +188,8 @@ def sage_setup(distributions, *,
     setup(cmdclass=cmdclass,
           packages=python_packages,
           py_modules=python_modules,
-          ext_modules=extensions,
+          data_files=data_files,
+          ext_modules=extensions + list(ext_modules),
+          options=options,
           **kwds
     )
